/**
 * Creates a Simple UDP Packet used to send a single, null-terminated String. 
 *  
 * @author Chadd Williams
 * 
 * @since 1.0
 *
 */

package edu.pacificu.cs.cs360.PUNetID.Lab.UDPLab;

/**
 * This class represents a Simple UDP Packet.
 * 
 * The packet layout is as follows (not to scale!):
 * 
 * <------- 4B -------><------- 4B ------->
 * +-------------------+------------------+
 * | Version           |      PayloadSize |
 * +---------+---------+---------+--------+ 
 * |                                      | 
 * : Payload                              :
 * | 0 - (1024-4-4)                       |
 * +--------------------------------------+
 * 
 * Strings in the payload MUST be NULL terminated
 */

public class SimpleUDPPacket {

	private int mVersion;
	private int mPayloadSize;
	private byte[] mPayload;

	public static final int VERSION_1 = 1;

	public static final int INT_SIZE = 4;

	public static final int MAX_PACKET_SIZE = 1024;
	public static final int MAX_PAYLOAD_SIZE = MAX_PACKET_SIZE - INT_SIZE
			- INT_SIZE;

	private byte[] rawPacket;

	
	/**
	 * Initializes mPayloadSize to zero
	 * 
	 * @since 1.0
	 */
	public SimpleUDPPacket() {
		mPayloadSize = 0;
	}


	/**
	 * This method must allocate the rawPacket array
	 * and fill the array with the relevant data from
	 * the member variables.
	 * after calling this function the packet is ready
	 * to send on the wire.
	 *
	 * @return the size of the byte array that represents the packet
	 * 
	 * @since 1.0
	 */
	public int buildPacket() {

		
		return 0;
	}

	/**
	 * Get a copy (a clone) of the byte array.
	 * 
	 * @return the byte array representing this packet.
	 * 
	 * @since 1.0
	 */
	public byte[] getPacket() {
		// return a new copy!
		return rawPacket.clone();
	}

	/**
	 * Parse the byte array parameter into the member variables
	 * in this packet.	 
	 *  
	 * 
	 * @param the byte array representing this packet.
	 * 
	 * @throws an exception if parsing fails
	 *  
	 * @since 1.0
	 */
	public void parsePacket(byte[] data) throws Exception {
		// read the data from the byte array and fill
		// the member variables.
		// use this after reading data from the wire.

	}

	/**
	 * Get the actual packet size.
	 * 
	 * @return the actual packet size.
	 * 
	 * @since 1.0
	 */
	public int getPacketSize() {
		return INT_SIZE + INT_SIZE + mPayloadSize;
	}

	/**
	 * Get the version.
	 * 
	 * @return the packet version
	 * 
	 * @since 1.0
	 */
	public int getVersion() {
		return mVersion;
	}


	/**
	 * Set the version.
	 * 
	 * @param the packet version
	 * 
	 * @since 1.0
	 */
	public void setVersion(int mVersion) {
		this.mVersion = mVersion;
	}

	/**
	 * Get the payloadSize
	 * 
	 * @return the payloadSize
	 * 
	 * @since 1.0
	 */
	public int getPayloadSize() {
		return mPayloadSize;
	}

	/**
	 * Set the payloadSize
	 * 
	 * @param the payloadSize
	 * 
	 * @since 1.0
	 */
	public void setPayloadSize(int mPayloadSize) {
		this.mPayloadSize = mPayloadSize;
	}
	
	/**
	 * Get the payload
	 * 
	 * @return the payload
	 * 
	 * @since 1.0
	 */
	public byte[] getPayload() {
		return mPayload.clone();
	}

	/**
	 * Set the payload
	 * 
	 * @param the payload
	 * 
	 * @since 1.0
	 */
	public void setPayload(byte[] mPayload) {
		this.mPayload = mPayload.clone();
	}
}
